package com.mso.master;


import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteStatement;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;

import com.google.android.gms.maps.model.LatLng;

public class DataManipulator{
	
   public final static  String DATABASE_NAME = "publictransport.db";
   public final String DATABASE_PATH = "/data/data/com.mso.master/databases/publictransport.db";
   private static final int DATABASE_VERSION = 1;
   static final String TABLE_NAME = "stops";
   static final String TABLE_NAME_ROUTES = "routes";
   static final String TABLE_NAME_POLYLINES = "polylines";
   static final String TABLE_NAME_BUSSTOPS = "busstops";
   static final String TABLE_NAME_DISTANCES = "distances";
   private static Context context;
   static SQLiteDatabase db;

   public boolean dbNotCreated = false;
   
   
   private SQLiteStatement insertStmt;
   private SQLiteStatement insertRouteStmt;
   private SQLiteStatement insertPolylineStmt;
   private SQLiteStatement insertBusStopStmt;
   private SQLiteStatement insertDistancesStmt;
    
   private static final String INSERT = "insert into " + TABLE_NAME + " (name,longitude,latitude) values (?,?,?)";
   private static final String INSERT_POLYLINE = "insert into " + TABLE_NAME_POLYLINES + " (latitude,longitude,route) values (?,?,?)";
   private static final String INSERT_BUSSTOP = "insert into " + TABLE_NAME_BUSSTOPS + " (name,abbname,locid,latitude,longitude) values (?,?,?,?,?)";
   private static final String INSERT_DISTANCE = "insert into " + TABLE_NAME_DISTANCES + " (name,distance,locid, loc) values (?,?,?,?)";
   
   public DataManipulator(Context context) {
	   DataManipulator.context = context;
	   OpenHelper openHelper = new OpenHelper(DataManipulator.context);

	   System.out.println(checkDataBase());
	   
	   if(!checkDataBase()){
		   DataManipulator.db = openHelper.getWritableDatabase();
		   createDatabase();
		   
		   this.insertStmt = DataManipulator.db.compileStatement(INSERT);
		   this.insertPolylineStmt = DataManipulator.db.compileStatement(INSERT_POLYLINE);
		   this.insertBusStopStmt = DataManipulator.db.compileStatement(INSERT_BUSSTOP);
		   this.insertDistancesStmt = DataManipulator.db.compileStatement(INSERT_DISTANCE);
		   
		   System.out.println("Placing bus stops in DB");
		   putBusStopsinDB();

	   }
	   else{
		   DataManipulator.db = openHelper.getWritableDatabase();
		   this.insertStmt = DataManipulator.db.compileStatement(INSERT);
		   this.insertPolylineStmt = DataManipulator.db.compileStatement(INSERT_POLYLINE);
		   this.insertBusStopStmt = DataManipulator.db.compileStatement(INSERT_BUSSTOP);
		   this.insertDistancesStmt = DataManipulator.db.compileStatement(INSERT_DISTANCE);
		   
	   }   
   }
   
   public long insert(String name, String longitude, String latitude) {
	   this.insertStmt.bindString(1, name);
	   this.insertStmt.bindString(2, longitude);
	   this.insertStmt.bindString(3, latitude);
	   return this.insertStmt.executeInsert();
   }

   public long insertPolylines(String latitude, String longitude, String route) {
	   this.insertPolylineStmt.bindString(1, latitude);
	   this.insertPolylineStmt.bindString(2, longitude);
	   this.insertPolylineStmt.bindString(3, route);
	   return this.insertPolylineStmt.executeInsert();
	   }
   
   public long insertBusStops(String name, String abbname, String locid, String latitude, String longitude){
	   this.insertBusStopStmt.bindString(1, name);
	   this.insertBusStopStmt.bindString(2, abbname);
	   this.insertBusStopStmt.bindString(3, locid);
	   this.insertBusStopStmt.bindString(4, latitude);
	   this.insertBusStopStmt.bindString(5, longitude);
	   return this.insertBusStopStmt.executeInsert();
   }
   
   public long insertDistance(String name, String distance, String locid, String loc){
	   this.insertDistancesStmt.bindString(1, name);
	   this.insertDistancesStmt.bindString(2, distance);
	   this.insertDistancesStmt.bindString(3, locid);
	   this.insertDistancesStmt.bindString(4, loc);
	   return this.insertDistancesStmt.executeInsert();
   }
   
   public void deleteAll() {
	   db.delete(TABLE_NAME, null, null);
	   db.delete(TABLE_NAME_ROUTES, null, null);
	   db.delete(TABLE_NAME_POLYLINES, null, null);
	   db.delete(TABLE_NAME_BUSSTOPS, null, null);
	   db.delete(TABLE_NAME_DISTANCES, null, null);
   }

   
//   public List<BusStop> selectAll() {
////   List<String[]> list = new ArrayList<String[]>();
//	 List<BusStop> list = new ArrayList<BusStop>();
//   Cursor cursor = db.query(TABLE_NAME, new String[] {"id","name","longitude","latitude"}, null, null, null, null, "name asc"); 
//   int x=0;
//   if (cursor.moveToFirst()) {
//      do {
//    	 BusStop stop = new BusStop();  
////    	 String[] b1=new String[]{cursor.getString(0),cursor.getString(1),cursor.getString(2),
////cursor.getString(3)};
//    	 stop.setBusStopId(Integer.parseInt(cursor.getString(0)));
//    	 stop.setName(cursor.getString(1));
//    	 stop.setLongitude(cursor.getString(2));
//    	 stop.setLatitude(cursor.getString(3));
//       
////       list.add(b1);
//    	 list.add(stop);
//       x=x+1;
//      } while (cursor.moveToNext());
//   }
//   if (cursor != null && !cursor.isClosed()) {
//      cursor.close();
//   } 
//   cursor.close();
//   return list;
//  }
   
   public List<DistanceBusStops> selectAllDistances() {
	   List<DistanceBusStops> list = new ArrayList<DistanceBusStops>();
	   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] { "id","name","distance","locid", "loc"}, null, null, null, null, "name asc"); 
	   int x=0;
	   if (cursor.moveToFirst()) {
	      do {
	    	 DistanceBusStops dist = new DistanceBusStops();  
	    	 dist.setBusStopName(cursor.getString(1));
	    	 dist.setDistance(cursor.getString(2));
	    	 dist.setId(cursor.getString(3));   
	    	 dist.setLoc(cursor.getString(4));
	    	 list.add(dist);
	       x=x+1;
	      } while (cursor.moveToNext());
	   }
	   if (cursor != null && !cursor.isClosed()) {
	      cursor.close();
	   } 
	   cursor.close();
	   return list;
	  }
   
   public List<String> selectDistanceById(String id){
		 List<String> list = new ArrayList<String>();
		   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] {"distance"}, "locid ="+"'"+id+"'", null, null, null, "distance asc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 list.add(cursor.getString(0));
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
 }
   
   public List<String> selectAllBusStopIdForDistanceTable(String loc){
		 List<String> list = new ArrayList<String>();
		   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] {"locid"}, "loc ="+"'"+loc+"'", null, null, null, "locid asc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 list.add(cursor.getString(0));
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
 }
   
   public List<DistanceBusStops> selectDistanceObjectByLoc(String loc){
		 List<DistanceBusStops> list = new ArrayList<DistanceBusStops>();
		   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] {"name", "distance", "locid", "loc"}, "loc ="+"'"+loc+"'", null, null, null, "loc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 DistanceBusStops dist = new DistanceBusStops();
		    	 dist.setBusStopName(cursor.getString(0));
			     dist.setDistance(cursor.getString(1));
			     dist.setId(cursor.getString(2));   
			     dist.setLoc(cursor.getString(3));
			     list.add(dist);
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
}
   
   public String checkIfDistanceTableHoldBusStopID(String locid){
		 String res = null;
		   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] {"name"}, "locid ="+"'"+locid+"'", null, null, null, null); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 res = cursor.getString(0);
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return res;
}
   
   public List<DistanceBusStops> selectDistanceObjectByLocID(String locid){
		 List<DistanceBusStops> list = new ArrayList<DistanceBusStops>();
		   Cursor cursor = db.query(TABLE_NAME_DISTANCES, new String[] {"name", "distance", "locid", "loc"}, "locid ="+"'"+locid+"'", null, null, null, "loc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 DistanceBusStops dist = new DistanceBusStops();
		    	 dist.setBusStopName(cursor.getString(0));
			     dist.setDistance(cursor.getString(1));
			     dist.setId(cursor.getString(2));   
			     dist.setLoc(cursor.getString(3));
			     list.add(dist);
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
}
   
   public List<BusStop> selectAllBusStops() {
	//   List<String[]> list = new ArrayList<String[]>();
		 List<BusStop> list = new ArrayList<BusStop>();
	   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] { "id","name","abbname","locid","latitude","longitude"}, null, null, null, null, "name asc"); 
	   int x=0;
	   if (cursor.moveToFirst()) {
	      do {
	    	 BusStop stop = new BusStop();  
	    	 stop.setName(cursor.getString(1));
	    	 stop.setNameWithAbbreviations(cursor.getString(2));
	    	 stop.setLocationId(cursor.getString(3));
	    	 stop.setLatitude(cursor.getString(4));
	    	 stop.setLongitude(cursor.getString(5));    	
	    	 list.add(stop);
	       x=x+1;
	      } while (cursor.moveToNext());
	   }
	   if (cursor != null && !cursor.isClosed()) {
	      cursor.close();
	   } 
	   cursor.close();
	   return list;
	  }
   
   
   public List<String> selectBusStopNames() {
	//   List<String[]> list = new ArrayList<String[]>();
		 List<String> list = new ArrayList<String>();
	   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] {"name"}, null, null, null, null, "name asc"); 
	   int x=0;
	   if (cursor.moveToFirst()) {
	      do {
	    	 list.add(cursor.getString(0));
	       x=x+1;
	      } while (cursor.moveToNext());
	   }
	   if (cursor != null && !cursor.isClosed()) {
	      cursor.close();
	   } 
	   cursor.close();
	   return list;
	  }
   
   
   public List<String> selectBusStopIDbyName(String name){
		 List<String> list = new ArrayList<String>();
		   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] {"locid"}, "name ="+"'"+name+"'", null, null, null, "locid asc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 list.add(cursor.getString(0));
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
   }
   
   public List<String> selectBusStopNameById(String id){
		 List<String> list = new ArrayList<String>();
		   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] {"name"}, "locid ="+"'"+id+"'", null, null, null, "name asc"); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 list.add(cursor.getString(0));
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return list;
 }
   
   public LatLng selectBusStopLocById(String id){
		 LatLng pos = null;
		   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] {"latitude", "longitude"}, "locid ="+"'"+id+"'", null, null, null, null); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 pos = new LatLng(Double.parseDouble(cursor.getString(0)), Double.parseDouble(cursor.getString(1)));
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return pos;
 }
   
   
   public String selectBusStopLocidByLocation(LatLng location){
		 String id = null;
		   Cursor cursor = db.query(TABLE_NAME_BUSSTOPS, new String[] {"locid"}, "latitude ="+"'"+location.latitude+"'"+ " AND " + "longitude ="+"'"+location.longitude+"'", null, null, null, null); 
		   int x=0;
		   if (cursor.moveToFirst()) {
		      do {
		    	 id = cursor.getString(0);
		       x=x+1;
		      } while (cursor.moveToNext());
		   }
		   if (cursor != null && !cursor.isClosed()) {
		      cursor.close();
		   } 
		   cursor.close();
		   return id;
 }
   

   
   

   
   public List<LatLng> selectPolylines(String busroute)
   {
//   List<String[]> list = new ArrayList<String[]>();
	 List<LatLng> list = new ArrayList<LatLng>();
   Cursor cursor = db.query(TABLE_NAME_POLYLINES, new String[] {"id","latitude","longitude","route"}, "route ="+"'"+busroute+"'", null, null, null, null); 
   int x=0;
   if (cursor.moveToFirst()) {
      do {  
    	  double t1 = Double.parseDouble(cursor.getString(1));
    	  double t2 = Double.parseDouble(cursor.getString(2));
    	  
    	  LatLng poly = new LatLng(t1,t2);

    	 list.add(poly);
       x=x+1;
      } while (cursor.moveToNext());
   }
   if (cursor != null && !cursor.isClosed()) {
      cursor.close();
   } 
   cursor.close();
   return list;
  }
   
   
   
  public void delete(int rowId) {
   db.delete(TABLE_NAME, null, null); 
   db.delete(TABLE_NAME_ROUTES, null, null); 
   db.delete(TABLE_NAME_POLYLINES, null, null); 
   db.delete(TABLE_NAME_BUSSTOPS, null, null); 
  }
  
  public void deleteDistanceTable(){
	  db.delete(TABLE_NAME_DISTANCES, null, null); 
  }
  
  public void putBusStopsinDB (){	
	  List<BusStop> busStopList = new ArrayList<BusStop>();
		try {    
		    InputStream file = context.getAssets().open("busstoplist.xls");

		    //Get the workbook instance for XLS file 
		    HSSFWorkbook workbook = new HSSFWorkbook(file);
		 
		    //Get first sheet from the workbook
		    HSSFSheet sheet = workbook.getSheetAt(0);
		     
		    //Iterate through each rows from first sheet
		    Iterator<Row> rowIterator = sheet.iterator();
		    
		    while(rowIterator.hasNext()) {
		  		BusStop busstop = new BusStop();
		        Row row = rowIterator.next();
	 
		        //For each row, iterate through each columns
		        Iterator<Cell> cellIterator = row.cellIterator();
		        while(cellIterator.hasNext()) {
		             
		            Cell cell = cellIterator.next();
		            
		           //add values to a busstop object
		            if(cell.getColumnIndex()==0){
		            	busstop.setName(cell.getStringCellValue());
		            }
		            else if(cell.getColumnIndex()==1){
//		            	String temp = cell.getStringCellValue();
//		            	Double f = Double.parseDouble(temp);
//		            	busstop.setLongitude(f);
		            	busstop.setLongitude(cell.getStringCellValue());
		            }
		            else if(cell.getColumnIndex()==2){
//		            	String temp = cell.getStringCellValue();
//		            	Double f = Double.parseDouble(temp);
//		            	busstop.setLatitude(f);
		            	busstop.setLatitude(cell.getStringCellValue());
		            }
		        }
//		        busStopList.add(busstop);
		        insert(busstop.getName(), busstop.getLongitude(), busstop.getLatitude());
		    }
		    file.close();     
		} catch (FileNotFoundException e) {
		    e.printStackTrace();
		} catch (IOException e) {
		    e.printStackTrace();
		}	
	}
  
  
  public boolean dbHoldsValues(){
	
	  List<BusStop> list = selectAllBusStops();
	  if(list.size() > 0){
		  return true;
	  }

	  return false;
	  
  }
  
  public boolean checkDataBase() {
	    SQLiteDatabase checkDB = null;
	    try {
	        checkDB = SQLiteDatabase.openDatabase(DATABASE_PATH, null,
	                SQLiteDatabase.OPEN_READONLY);
	        checkDB.close();
	    } catch (SQLiteException e) {
	        // database doesn't exist yet.
	    }
	    return checkDB != null ? true : false;
	}

  public void createDistanceTable(){
	  System.out.println("Creating distance table in DB");
      db.execSQL("CREATE TABLE " + TABLE_NAME_DISTANCES + " (id INTEGER PRIMARY KEY, name TEXT, distance TEXT, locid TEXT)");
  }
  
  public void createDatabase(){
	  System.out.println("Creating Database");
      db.execSQL("CREATE TABLE " + TABLE_NAME + " (id INTEGER PRIMARY KEY, name TEXT, longitude TEXT, latitude TEXT)");
      db.execSQL("CREATE TABLE " + TABLE_NAME_POLYLINES + " (id INTEGER PRIMARY KEY,  latitude TEXT, longitude TEXT, route TEXT)");
      db.execSQL("CREATE TABLE " + TABLE_NAME_BUSSTOPS + " (id INTEGER PRIMARY KEY,  name TEXT, abbname TEXT, locid TEXT, latitude TEXT, longitude TEXT)");
      db.execSQL("CREATE TABLE " + TABLE_NAME_DISTANCES + " (id INTEGER PRIMARY KEY, name TEXT, distance TEXT, locid TEXT, loc TEXT)");
      System.out.println("All three tables created in database");
  }
  
  
  private static class OpenHelper extends SQLiteOpenHelper {
	 
	  
   OpenHelper(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
   }
   
   @Override
   public void onCreate(SQLiteDatabase db) {
//	   System.out.println("Creating Database");
//       db.execSQL("CREATE TABLE " + TABLE_NAME + " (id INTEGER PRIMARY KEY, name TEXT, longitude TEXT, latitude TEXT)");
//       db.execSQL("CREATE TABLE " + TABLE_NAME_ROUTES + " (id INTEGER PRIMARY KEY, route TEXT, busstop TEXT, longitude TEXT, latitude TEXT)");
//       db.execSQL("CREATE TABLE " + TABLE_NAME_POLYLINES+ " (id INTEGER PRIMARY KEY,  latitude REAL, longitude REAL)");
//       System.out.println("All three tables created in database");
   }
   

   @Override
   public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) 
       {
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_NAME);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_NAME_POLYLINES);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_NAME_BUSSTOPS);
        onCreate(db);
   }
  }
}